//
//***************************************************************************
// Name    : main_hid_SFtest.bas                                            *
// Author  : Jerry Messina                                                  *
// Notice  : Copyright (c) 20011                                            *
//         : All Rights Reserved                                            *
// Date    : 4/7/2011                                                       *
// Version : 1.1                                                            *
// Notes   : USB HID main application example from the SF USB HID Library   *
//         : help example. You can use the test HID executable available    *
//         : at http://www.sfcompiler.co.uk/wiki/uploads/HIDExample.zip to  *
//         : interface with this program                                    *
// Ver 1.1 : added 'SF startup initialization' section to ensure register   *
//         : settings at poweron when used with a bootloader                *
//***************************************************************************
//

// Sparkfun USB whacker hardware
device = 18F2553
clock = 48

config
    FOSC = HSPLL_HS,         ' External 24MHz xtal
    PLLDIV = 6,
    CPUDIV = OSC1_PLL2,      ' 48MHz cpu clock
    USBDIV = 2,              ' 48MHz usb clock
    FCMEN = OFF,
    IESO = OFF,
    PWRT = ON,
    BOR = OFF,
    BORV = 2,
    VREGEN = ON,
    WDT = OFF,
    WDTPS = 2048,
    CCP2MX = OFF,
    PBADEN = OFF,
    LPT1OSC = OFF,
    MCLRE = ON,
    STVREN = ON,
    LVP = OFF,
    XINST = OFF,
    DEBUG = OFF

//
//-----------------------------------------------------------------------------
// enable display of status messages to the Results window
//-----------------------------------------------------------------------------
//
#option SHOW_USB_BUILD_MSGS = true      // for usb modules (off by default)

//
//-----------------------------------------------------------------------------
// program build options
//-----------------------------------------------------------------------------
//
// build for bootloader/standalone operation
#variable BUILD_FOR_BOOTLOADER = false
#variable APP_PROGRAM_START = $001000       // must agree with bootloader setting

#if (BUILD_FOR_BOOTLOADER = true)           // build for use with bootloader
  #option org_reset = APP_PROGRAM_START
  #warning "building for use with bootloader"
#endif

//
//-----------------------------------------------------------------------------
// SF startup initialization
//-----------------------------------------------------------------------------
// note: include system.bas first before any other files
// this is VERY important if you're using a bootloader (requires system.bas V1.4 or later)
// be sure to set any system.bas WDT options prior to including the file
'#option WDT = true
include "system.bas"
#if not defined(SF_SYSTEM_VERSION) or (SF_SYSTEM_VERSION < 14)
  #if (BUILD_FOR_BOOTLOADER = true)
    #error "bootloader use requires system.bas v1.4 or later"
  #endif
#endif

//
//-------------------------------------------------------------
// USB options and includes
//-------------------------------------------------------------
//
// import modules...
include "usbhid.bas"

//
//-------------------------------------------------------------
// USB report structures
//-------------------------------------------------------------
//
// TX report...
structure TTXReport
   Time as word
   Message as string
end structure

dim TXReport as TTXReport absolute TXReportRAM

// RX report...
structure TRXReport
   LED0 as bit
   LED1 as bit
end structure

dim RXReport as TRXReport absolute RXReportRAM

//
//-------------------------------------------------------------
// main program
//-------------------------------------------------------------
//
// alias port pins to LEDs...
// usb whacker hardware (18F2553)
public dim YELLOW_LED  as PORTC.0     // usb whacker LED2
public dim RED_LED     as PORTC.1     // usb whacker LED3

dim
   LED0 as YELLOW_LED,
   LED1 as RED_LED

// initialise...
TXReport.Time = 0
low(LED0)
low(LED1)

high(LED0)
// connect to USB...
repeat
until Attached
low(LED0)

// main program loop...
while true
   // if we have data, set port values, update message
   // and then reset time counter...
   if DataAvailable then
      ReadReport
      LED0 = RXReport.LED0
      LED1 = RXReport.LED1
      TXReport.Message = "PORT CHANGED"
      WriteReport
      TXReport.Time = 0
      delayms(100)
   // no data, set waiting message...   
   else
      TXReport.Message = "WAITING..."
      WriteReport
      inc(TXReport.Time)
   endif
end while



